/*
 8x8 x10 LED Scrolling Receiver
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 11/03/2015
 Last Modified: 28/05/2015
 
 Wait to receive wake up command / program command and execute command then scroll the text received.
 
 */

// ATmega328 Pin Map
// 
//                 +-\/-+
//     Reset PC6  1|o   |28  PC5
//  ULN COL8 PD0  2|    |27  PC4 
//  ULN COL7 PD1  3|    |26  PC3 
//  ULN COL6 PD2  4|    |25  PC2 Latch
//  ULN COL5 PD3  5|    |24  PC1 Data
//  ULN COL4 PD4  6|    |23  PC0 Clock
//           VCC  7|    |22  GND
//           GND  8|    |21  AREF
//           PB6  9|    |20  AVCC
//           PB7 10|    |19  PB5 SCK
//  ULN COL3 PD5 11|    |18  PB4 MO
//  ULN COL2 PD6 12|    |17  PB3 MI
//  ULN COL1 PD7 13|    |16  PB2 CSN
//           PB0 14|    |15  PB1 CE
//                 +----+

#define F_CPU 8000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"

uint8_t data_in[32];
uint8_t ledLocation = 0; // Where in the leds animation array we are
uint8_t ledMsgSize = 0; // Text string length
uint8_t ledSpeed = 0; // Scrolling speed
uint8_t ledsMessage[256];
uint8_t LEDs[8][88];
volatile uint8_t timedOut = false;

// Shift out to 74HC595 shift registers
void shift_out(uint8_t data) {
	for (int8_t x = 7; x >= 0; x--) {
		if (data & (1<<x)) {
			PORTC |= (1<<PC1);
		}
		else {
			PORTC &= ~(1<<PC1);
		}
		PORTC |= (1<<PC0); // CLK
		PORTC &= ~(1<<PC0);
	}
}

// Move all LEDs left by one
void move_LEDs(void) {
	for (uint8_t x = 0; x < 8; x++) {
		for (uint8_t y = 0; y < 87; y++) {
			LEDs[x][y] = LEDs[x][y+1];
		}
	}
}

// Clear all LEDs
void clear_LEDs(void) {
	for (uint8_t x = 0; x < 8; x++) {
		for (uint8_t y = 0; y < 88; y++) {
			LEDs[x][y] = 0;
		}
	}
}

// Light up all LEDs
void light_LEDs(void) {
	for (int8_t x = 0; x < 8; x++) {
		for (uint8_t matrix = 0; matrix < 10; matrix++) {
			uint8_t bitsToSend = 0;
			for (uint8_t y = 0; y < 8; y++) {
				if (LEDs[y][x+(matrix*8)] == 1) {
				  bitsToSend |= (1<<y);
				}
			}
			shift_out(bitsToSend);
		}
		
		PORTD = 0; // Turn all columns off
		PORTC |= (1<<PC2); // Latch
		PORTC &= ~(1<<PC2); // Latch
		PORTD = (1<<x);
		
		_delay_ms(1);
	}
}

// Append to the end 8x8 of the 8x16 matrix
void append_text(void) {
	uint8_t letter = (uint8_t) ledsMessage[ledLocation];
	
	if (letter == 32) { // Space
		letter = 26;
	}
	else if (letter == 33) { // ?
		letter = 27;
	}
	else if (letter == 63) { // !
		letter = 28;
	}
	else if (letter == 46) { // .
		letter = 29;
	}
	else if (letter >= 97) { // If it's a lowercase letter, remove 97 from it to line up with text to led letter
		letter-= 97;
	}
	else if (letter >= 65 || letter <= 90) { // If it's a capital letter
		letter-= 65;
	}
	
	// Insert text
	for (uint8_t row = 0; row < 8; row++) {
		uint8_t data = (uint8_t) pgm_read_byte(&ledLetters[letter][row]);
		
		for (uint8_t column = 0; column < 8; column++) {
			if (data & (1<<column)) {
				LEDs[row][87-column] = 1;
			}
			else {
				LEDs[row][87-column] = 0;
			}
		}
	}	
	ledLocation++;
}

int main(void) {
	setup();
	
	while (1) {
		// Listen for incoming packets for 10ms
		if (mirf_listen()) {
			
			// Blink
			LEDs[0][8] = 1;
			for (uint8_t i = 0; i < 10; i++) {
				light_LEDs();
			}
			LEDs[0][8] = 0;
			
			// Wait for new packets to arrive and received execute command
			if (mirf_receive_data()) {
				clear_LEDs();
				ledLocation = 0;
				for (uint8_t x = 0; x < ledMsgSize; x++) {
					// Append the matrix with the next text
					append_text();
					
					// Move the matrix to the left 8 times
					for (uint8_t x = 0; x < 8; x++) {
						move_LEDs();
						// Light up the LEDs a few times to fill in time
						for (uint8_t i = 0; i < ledSpeed; i++) {
							light_LEDs();
						}
					}
				}
				clear_LEDs();
			}
		}
		else {
			// Blink
			LEDs[0][0] = 1;
			for (uint8_t i = 0; i < 10; i++) {
				light_LEDs();
			}
			LEDs[0][0] = 0;
			
			// Sleep for 1 second
			watchdog_sleep(T1S);
		}
	}
}

ISR(WDT_vect) { }

// Timer1 overflow after 8 seconds
ISR(TIMER1_OVF_vect) {
	timedOut = true;
}