/*
 8x8 x10 LED Scrolling Transmitter
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 11/03/2015
 Last Modified: 28/05/2015
 
 Read the text using V-USB and transmit the text via nRF24L01 to the LED scrolling receiver.
 Send wake up command for about 1 second, send the text to program and then execute.
 
 Remember to change your fuse bits so the ATtiny44A uses 16MHz crystal:
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0xff:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny24/44/84 Pin map
//
//                     +-\/-+
//               VCC  1|o   |14  GND
//          XTAL PB0  2|    |13  PA0 USB D+ 
//          XTAL PB1  3|    |12  PA1 USB D-
//         Reset PB3  4|    |11  PA2
//        USB D+ PB2  5|    |10  PA3 nRF24L01 CE
//  nRF24L01 CSN PA7  6|    |9   PA4 nRF24L01 SCK
//   nRF24L01 MI PA6  7|    |8   PA5 nRF24L01 MO
//                     +----+

#define F_CPU 16000000 // If this will be changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"
#include "mirf.c"

uint8_t data_out[32];
uint8_t send_text = false;
uint8_t text_size = 0;

// USB transfer variables
#define USB_TEXT 2
uchar replyBuf[254];
static uchar dataReceived = 0, dataLength = 0; // for USB_DELAYTIME

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TEXT) { // Re-program the delay time
		dataLength = (uchar)rq->wLength.word;
		dataReceived = 0;
		
		if (dataLength > sizeof(replyBuf)) { // Limit to buffer size
			dataLength = sizeof(replyBuf);
		}
		
		return USB_NO_MSG; // usbFunctionWrite will be called now
	}
	
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {	
	for (uchar i = 0; dataReceived < dataLength && i < len; i++, dataReceived++) {
		replyBuf[dataReceived] = data[i];
	}
	
	// Once all data is received
	if (dataReceived == dataLength) {
		send_text = true;
	}
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

int main (void) {
	setup();
	startUSB();
	
	while(1) {
		usbPoll();
		
		if (send_text == true) {
			// Wake up LED text
			text_size = replyBuf[0];
			mirf_send_wakeup(text_size, 8);
			
			// Wait a little bit before sending the text data
			_delay_ms(1000);
			mirf_transmit_data();
			
			// Execute the text
			_delay_ms(500);
			mirf_send_execute();
			send_text = false;
		}
	}
	
	return 0;
}
