/*
 8x8 x10 LED Scrolling Transmitter
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 11/03/2015
 Last Modified: 28/05/2015
  
 */
 
/*	Copyright (c) 2007 Stefan Engelke <mbox@stefanengelke.de>
	
	Permission is hereby granted, free of charge, to any person 
	obtaining a copy of this software and associated documentation 
	files (the "Software"), to deal in the Software without 
	restriction, including without limitation the rights to use, copy, 
	modify, merge, publish, distribute, sublicense, and/or sell copies 
	of the Software, and to permit persons to whom the Software is 
	furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be 
	included in all copies or substantial portions of the Software.
	
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
	DEALINGS IN THE SOFTWARE.
	
	$Id$
*/

#include "mirf.h"
#include "nRF24L01.h"
#include <avr/io.h>
#include <avr/interrupt.h>

// Defines for setting the MiRF registers for transmitting or receiving mode
#define TX_POWERUP mirf_config_register(CONFIG, mirf_CONFIG | ( (1<<PWR_UP) | (0<<PRIM_RX) ) )
#define RX_POWERUP mirf_config_register(CONFIG, mirf_CONFIG | ( (1<<PWR_UP) | (1<<PRIM_RX) ) )
#define POWERDOWN mirf_config_register(CONFIG, mirf_CONFIG | ( (0<<PWR_UP) ) )

extern uint8_t data_out[];
extern uint8_t replyBuf[];
extern uint8_t text_size;

// Initializes pins ans interrupt to communicate with the MiRF module
// Should be called in the early initializing phase at startup.
void mirf_init(void) {
	// Define CSN and CE as Output and set them to default
	DDRA |= (1<<CE);
	DDRA |= (1<<CSN);
	mirf_CE_lo;
	mirf_CSN_hi;
}

// Sets the important registers in the MiRF module
void mirf_config(void) {
	// Set RF channel
	mirf_config_register(RF_CH, mirf_CH);
	
	// Set length of incoming payload 
	mirf_config_register(RX_PW_P0, mirf_PAYLOAD);
	mirf_config_register(RX_PW_P1, mirf_PAYLOAD);
	
	// Set auto retransmit delay to 500uS and leave auto retransmit count to 3
	mirf_config_register(SETUP_RETR, (1<<4) | 3);
	
	// Set RADDR and TADDR
	mirf_write_register(RX_ADDR_P0, TADDR, 5);
	mirf_write_register(RX_ADDR_P1, RADDR, 5);
	mirf_write_register(TX_ADDR, TADDR, 5);
	
	// Enable RX_ADDR_P0 and RX_ADDR_P1 address matching since we also enable auto acknowledgement
	mirf_config_register(EN_RXADDR, (1<<ERX_P0 | 1<<ERX_P1));
}

// Flush RX and TX FIFO
void mirf_flush_rx_tx(void) {
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(FLUSH_RX); // Flush RX
	mirf_CSN_hi; // Pull up chip select

	mirf_CSN_lo; // Pull down chip select
	spi_transfer(FLUSH_TX);  // Write cmd to flush tx fifo
	mirf_CSN_hi; // Pull up chip select
}

// Read the status register
uint8_t mirf_status(void) {
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(R_REGISTER | (REGISTER_MASK & STATUS));
	uint8_t status = spi_transfer(NOP); // Read status register
	mirf_CSN_hi; // Pull up chip select
	return status;
}

// Checks if data is available for reading
uint8_t mirf_data_ready(void) {
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(R_REGISTER | (REGISTER_MASK & STATUS));
	uint8_t status = spi_transfer(NOP); // Read status register
	mirf_CSN_hi; // Pull up chip select
	return status & (1<<RX_DR);
}

// Checks if MAX_RT has been reached
uint8_t mirf_max_rt_reached(void) {
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(R_REGISTER | (REGISTER_MASK & STATUS));
	uint8_t status = spi_transfer(NOP); // Read status register
	mirf_CSN_hi; // Pull up chip select
	return status & (1<<MAX_RT);
}

// Reads mirf_PAYLOAD bytes into data array
void mirf_get_data(uint8_t *data) {
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
	spi_read_data(data, mirf_PAYLOAD); // Read payload
	mirf_CSN_hi; // Pull up chip select
	mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
}

// Write one byte into the MiRF register
void mirf_config_register(uint8_t reg, uint8_t value) {
	mirf_CSN_lo;
	spi_transfer(W_REGISTER | (REGISTER_MASK & reg));
	spi_transfer(value);
	mirf_CSN_hi;
}

// Reads an array of bytes from the MiRF registers.
void mirf_read_register(uint8_t reg, uint8_t *value, uint8_t len) {
	mirf_CSN_lo;
	spi_transfer(R_REGISTER | (REGISTER_MASK & reg));
	spi_read_data(value, len);
	mirf_CSN_hi;
}

// Writes an array of bytes into the MiRF register
void mirf_write_register(uint8_t reg, uint8_t *value, uint8_t len) {
	mirf_CSN_lo;
	spi_transfer(W_REGISTER | (REGISTER_MASK & reg));
	spi_write_data(value, len);
	mirf_CSN_hi;
}

// Send wake up with message size and speed
void mirf_send_wakeup(uint8_t msgsize, uint8_t speed) {
	// Clear the data_out
	for (uint8_t x = 0; x < mirf_PAYLOAD; x++) {
		data_out[x] = 0; 
	}
	
	data_out[ACTION] = WAKEUP;
	data_out[MSG_SIZE] = msgsize;
	data_out[SPEED] = speed;
	
	TX_POWERUP; // Power up to transmitter mode
	_delay_ms(3);
	mirf_flush_rx_tx(); // Flush RX and TX FIFO
	mirf_config_register(STATUS, 1<<MAX_RT);
	
	mirf_CSN_lo;  // Pull down chip select
	spi_transfer(W_TX_PAYLOAD); // Write cmd to write payload
	spi_write_data(data_out, mirf_PAYLOAD); // Write payload
	mirf_CSN_hi; // Pull up chip select
	
	mirf_CSN_lo; // Pull down chip select
	spi_transfer(REUSE_TX_PL); // Reuse TX
	mirf_CSN_hi; // Pull up chip select
	
	// Send for about ~1 second
	for (uint8_t x = 0; x < 255; x++) {
		mirf_CE_hi; // Start transmission
		_delay_us(15);
		mirf_CE_lo;
		_delay_ms(5); // Wait for transmission/re-transmit
		
		mirf_config_register(STATUS, 1<<MAX_RT);
	}
	
	POWERDOWN;
}

// Send execute command
void mirf_send_execute(void) {
	// Clear the data_out
	for (uint8_t x = 0; x < mirf_PAYLOAD; x++) {
		data_out[x] = 0; 
	}
	
	data_out[ACTION] = EXECUTE;
	
	TX_POWERUP; // Power up to transmitter mode
	_delay_ms(3);
	mirf_flush_rx_tx(); // Flush RX and TX FIFO
	mirf_config_register(STATUS, 1<<MAX_RT);
	
	mirf_CSN_lo;  // Pull down chip select
	spi_transfer(W_TX_PAYLOAD); // Write cmd to write payload
	spi_write_data(data_out, mirf_PAYLOAD); // Write payload
	mirf_CSN_hi; // Pull up chip select
	
	mirf_CE_hi; // Start transmission
	_delay_us(15);
	mirf_CE_lo;
	_delay_ms(2); // Wait for transmission/re-transmit
	
	POWERDOWN;
}

// Send data
uint8_t mirf_transmit_data(void) {
	// Clear the data_out
	for (uint8_t x = 0; x < mirf_PAYLOAD; x++) {
		data_out[x] = 0; 
	}
	
	TX_POWERUP; // Power up to transmitter mode
	_delay_ms(3);
	mirf_flush_rx_tx(); // Flush RX and TX FIFO
	mirf_config_register(STATUS, 1<<MAX_RT);
	
	uint8_t ledMsgCounter = 1;
	data_out[ACTION] = PROGRAM;
	while (ledMsgCounter < text_size) {
		for (uint8_t x = 1; x < mirf_PAYLOAD; x++) {
			data_out[x] = replyBuf[ledMsgCounter];
			ledMsgCounter++;
		}
		
		mirf_CSN_lo;  // Pull down chip select
		spi_transfer(W_TX_PAYLOAD); // Write cmd to write payload
		spi_write_data(data_out, mirf_PAYLOAD); // Write payload
		mirf_CSN_hi; // Pull up chip select
		
		mirf_CE_hi; // Start transmission
		_delay_us(15);
		mirf_CE_lo;
		_delay_ms(2); // Wait for transmission/re-transmit/receiver to read out data
		
		mirf_config_register(STATUS, 1<<MAX_RT); // Reset max retries 
	}
	
	POWERDOWN;
	
	return 1;
}
